@@ -3,7 +3,7 @@
 # abstract:  Bootstrapping Compiler for a Pegex Grammar
 # author:    Ingy döt Net <ingy@cpan.org>
 # license:   perl
-# copyright: 2010, 2011
+# copyright: 2010, 2011, 2012
 
 package Pegex::Bootstrap;
 use Pegex::Mo;
@@ -11,38 +11,74 @@ extends 'Pegex::Compiler';
 
 use Pegex::Grammar::Atoms;
 
+my $modifier = qr{[\!\=\-\+\.]};
+my $group_modifier = qr{[\.]};
 my $quantifier = qr{(?:[\?\*\+]|\d+(?:\+|\-\d+)?)};
 
 sub parse {
-    my $self = shift;
+    my ($self, $grammar_text) = @_;
     $self = $self->new unless ref $self;
-    my $grammar_text = shift;
-    if ($grammar_text !~ /[\n\:]/) {
+
+    # If the grammar looks like a filename, try to read that file for the
+    # grammar content.
+    if (length($grammar_text) and $grammar_text !~ /(\s|\:|\#|\%)/) {
         open IN, $grammar_text
             or die "Can't open file '$grammar_text' for input";
         $grammar_text = do {local $/; <IN>};
         close IN;
     }
     $self->tree({});
+
+    # Remove comment lines
     $grammar_text =~ s/^#.*\n+//gm;
-    $grammar_text =~ s/^\s*\n//;
-    $grammar_text .= "\n" unless $grammar_text =~ /\n\z/;
+
+    # Remove trailing comments
+    $grammar_text =~ s/\ *#.*//g;
+
+    # Remove blank lines
+    $grammar_text =~ s/^\s*\n//gm;
+
+    # Turn semis into line breaks
     $grammar_text =~ s/;/\n/g;
+
+    # Ensure trailing newline
+    $grammar_text .= "\n" unless
+        $grammar_text eq '' or
+        $grammar_text =~ /\n\z/;
+
+    # Process directives
     if ($grammar_text =~ s/\A((%\w+ +.*\n)+)//) {
         my $section = $1;
-        my (%directives) = ($section =~ /%(\w+) +(.*?) *\n/g);
-        for my $key (keys %directives) {
-            $self->tree->{"+$key"} = $directives{$key};
+        my (@directives) = ($section =~ /%(\w+) +(.*?) *\n/g);
+        my $tree = $self->tree;
+        while (@directives) {
+            my ($key, $val) = splice(@directives, 0, 2);
+            die "'$key' is an invalid Pegex directive"
+                unless $key =~ /^(grammar|version|extends|include)$/;
+            $key = "+$key";
+            my $old = $tree->{$key};
+            if (defined $old) {
+                if (ref $old) {
+                    push @$old, $val;
+                }
+                else {
+                    $tree->{$key} = [ $old, $val ];
+                }
+            }
+            else {
+                $tree->{$key} = $val;
+            }
         }
     }
+
     for my $rule (split /(?=^\w+:\s*)/m, $grammar_text) {
         (my $value = $rule) =~ s/^(\w+):// or die "$rule";
         my $key = $1;
         $value =~ s/\s+/ /g;
         $value =~ s/^\s*(.*?)\s*$/$1/;
         $self->tree->{$key} = $value;
-        $self->tree->{'+top'} ||= $key;
-        $self->tree->{'+top'} = $key if $key eq 'TOP';
+        $self->tree->{'+toprule'} ||= $key;
+        $self->tree->{'+toprule'} = $key if $key eq 'TOP';
     }
 
     for my $rule (sort keys %{$self->tree}) {
@@ -51,35 +87,36 @@ sub parse {
         my @tokens = grep $_,
         ($text =~ m{(
             /[^/\n]*/ |
-            %?%%? |
-            [\!\=\-\+\.]?<\w+>$quantifier? |
+            ~+ |
+            %%? |
+            $modifier?<\w+>$quantifier? |
+            $modifier?\w+$quantifier? |
             `[^`\n]*` |
             \| |
-            [\!\=?\.]?\[ |
-            \]$quantifier? |
+            $group_modifier?\( |
+            \)$quantifier? |
         )}gx);
         die "No tokens found for rule <$rule> => '$text'"
             unless @tokens;
-        unshift @tokens, '[';
-        push @tokens, ']';
+        unshift @tokens, '(';
+        push @tokens, ')';
         my $tree = $self->make_tree(\@tokens);
-        $self->tree->{$rule} = $self->compile_next($tree);  
+        $self->tree->{$rule} = $self->compile_next($tree);
     }
     return $self;
 }
 
 sub make_tree {
-    my $self = shift;
-    my $tokens = shift;
+    my ($self, $tokens) = @_;
     my $stack = [];
     my $tree = [];
     push @$stack, $tree;
     for my $token (@$tokens) {
-        if ($token =~ /^[\!\=?\.]?[\[]/) {
+        if ($token =~ /^$group_modifier?\(/) {
             push @$stack, [];
         }
         push @{$stack->[-1]}, $token;
-        if ($token =~ /^[\]]/) {
+        if ($token =~ /^\)/) {
             my $branch = pop @$stack;
             push @{$stack->[-1]}, $self->wilt($branch);
         }
@@ -88,12 +125,11 @@ sub make_tree {
 }
 
 sub wilt {
-    my $self = shift;
-    my $branch = shift;
+    my ($self, $branch) = @_;
     return $branch unless ref($branch) eq 'ARRAY';
     my $wilted = [];
     for (my $i = 0; $i < @$branch; $i++) {
-        push @$wilted, ($branch->[$i] =~ /^%?%%?$/)
+        push @$wilted, ($branch->[$i] =~ /^%%?$/)
             ? [$branch->[$i], pop(@$wilted), $branch->[++$i]]
             : $branch->[$i];
     }
@@ -101,17 +137,19 @@ sub wilt {
 }
 
 sub compile_next {
-    my $self = shift;
-    my $node = shift;
+    my ($self, $node) = @_;
     my $unit = ref($node) ?
-        $node->[0] =~ /^%?%%?$/
+        $node->[0] =~ /^%%?$/
             ? $self->compile_sep($node) :
         $node->[2] eq '|'
             ? $self->compile_group($node, 'any')
             : $self->compile_group($node, 'all')
     :
+        $node =~ /^~+$/ ? $self->compile_ws($node) :
         $node =~ m!/! ? $self->compile_re($node) :
         $node =~ m!<! ? $self->compile_rule($node) :
+        $node =~ m!^$modifier?\w+$quantifier?$!
+            ? $self->compile_rule($node) :
         $node =~ m!`! ? $self->compile_error($node) :
             XXX $node;
 
@@ -129,28 +167,30 @@ my %prefixes = (
     '+' => '-wrap',
 );
 
+sub compile_ws {
+    my ($self, $node) = @_;
+    my $regex = '<ws' . length($node) . '>';
+    return { '.rgx' => $regex };
+}
+
 sub compile_sep {
-    my $self = shift;
-    my $node = shift;
+    my ($self, $node) = @_;
     my $object = $self->compile_next($node->[1]);
     $object->{'.sep'} = $self->compile_next($node->[2]);
-    $object->{'.sep'}{'+eok'} = 1 if $node->[0] =~ /^%?%%$/;
-    $object->{'.sep'}{'+bok'} = 1 if $node->[0] eq '%%%';
+    $object->{'.sep'}{'+eok'} = 1 if $node->[0] eq '%%';
     return $object;
 }
 
 sub compile_group {
-    my $self = shift;
-    my $node = shift;
-    my $type = shift;
+    my ($self, $node, $type) = @_;
     die unless @$node > 2;
     my $object = {};
-    if ($node->[0] =~ /^([\=\!\.\-\+])/) {
+    if ($node->[0] =~ /^($modifier)/) {
         my ($key, $val) = ($prefixes{$1}, 1);
         ($key, $val) = @$key if ref $key;
         $object->{$key} = $val;
     }
-    if ($node->[-1] =~ /([\?\*\+])$/) {
+    if ($node->[-1] =~ /($quantifier)$/) {
         $self->set_quantity($object, $1);
     }
     shift @$node;
@@ -194,19 +234,19 @@ sub set_quantity {
 }
 
 sub compile_re {
-    my $self = shift;
-    my $node = shift;
+    my ($self, $node) = @_;
     my $object = {};
     $node =~ s!^/(.*)/$!$1! or die $node;
+    $node =~ s!\s+!!g;
+    $node =~ s!\((\:|\=|\!)!(?$1!g;
     $object->{'.rgx'} = $node;
     return $object;
 }
 
 sub compile_rule {
-    my $self = shift;
-    my $node = shift;
+    my ($self, $node) = @_;
     my $object = {};
-    if ($node =~ s/^([\=\!\-\+\.])//) {
+    if ($node =~ s/^($modifier)//) {
         my ($key, $val) = ($prefixes{$1}, 1);
         ($key, $val) = @$key if ref $key;
         $object->{$key} = $val;
@@ -214,7 +254,7 @@ sub compile_rule {
     if ($node =~ s/($quantifier)$//) {
         $self->set_quantity($object, $1);
     }
-    $node =~ s!^<(.*)>$!$1! or XXX $node;
+    $node =~ s!^<(.*)>$!$1!;
     $object->{'.ref'} = $node;
     if (defined(my $re = Pegex::Grammar::Atoms->atoms->{$node})) {
         $self->tree->{$node} ||= {'.rgx' => $re};
@@ -223,8 +263,7 @@ sub compile_rule {
 }
 
 sub compile_error {
-    my $self = shift;
-    my $node = shift;
+    my ($self, $node) = @_;
     my $object = {};
     $node =~ s!^`(.*)`$!$1! or die $node;
     $object->{'.err'} = $node;
