@@ -3,7 +3,7 @@
 # abstract:  Pegex Regex Atoms
 # author:    Ingy döt Net <ingy@cpan.org>
 # license:   perl
-# copyright: 2011
+# copyright: 2011, 2012
 
 package Pegex::Grammar::Atoms;
 use Pegex::Mo;
@@ -12,7 +12,9 @@ use Pegex::Mo;
 # Pegex regex atoms for grammars
 #------------------------------------------------------------------------------#
 my $atoms = {
-    ws      => '<WS>*',
+    ws      => '<WS>',
+    ws1     => '<ws>*',
+    ws2     => '<ws>+',
     ALWAYS  => '',
     NEVER   => '(?!)',
     ALL     => '[\s\S]',
@@ -35,6 +37,7 @@ my $atoms = {
     XDIGIT  => '[0-9a-fA-F]',
     ALNUM   => '[a-zA-Z0-9]',
     WORD    => '\w',
+    HICHARS => '[\x7f-\uffff]',
 
     SINGLE  => "'",
     DOUBLE  => '"',
@@ -56,7 +59,7 @@ my $atoms = {
     AMP     => '&',
     STAR    => '\*',
 
-    TILDE   => '~',
+    TILDE   => '\~',
     GRAVE   => '`',
     UNDER   => '_',
     DASH    => '\-',
