@@ -3,76 +3,96 @@
 # abstract:  Pegex Parser Runtime
 # author:    Ingy döt Net <ingy@cpan.org>
 # license:   perl
-# copyright: 2011
+# copyright: 2011, 2012
 # see:
 # - Pegex::Grammar
+# - Pegex::Receiver
 
 package Pegex::Parser;
 use Pegex::Mo;
 
-use Pegex::Input;
-
 use Scalar::Util;
+use Pegex::Input;
 
 # Grammar object or class
 has 'grammar';
 # Receiver object or class
-has 'receiver' => default => sub {
-    require Pegex::Receiver;
-    Pegex::Receiver->new();
-};
+has 'receiver' => (
+    default => sub {
+        require Pegex::Receiver;
+        Pegex::Receiver->new();
+    },
+);
 
+#
 # Parser options
-has 'throw_on_error' => default => sub {1};
+#
+
+# Allow errors to not be thrown
+has 'throw_on_error' => ( default => sub {1} );
+
+# Wrap results in hash with rule name for key
+has 'wrap' => ( default => sub { $_[0]->receiver->wrap } );
+
 # # Allow a partial parse
 # has 'partial' => default => sub {0};
-# Wrap results in hash with rule name for key
-has 'wrap' => default => sub { $_[0]->receiver->wrap };
 
 # Internal properties.
-has 'input';
-has 'buffer';
-has 'position' => default => sub {0};
+has 'input';                # Input object to read from
+has 'buffer';               # Input buffer to parse
+has 'error';                # Error message goes here
+has 'position' => (         # Current position in buffer
+    default => sub {0},
+);
+has 'farthest' => (         # Farthest point matched in buffer
+    default => sub {0},
+);
 
 # Debug the parsing of input.
-has 'debug' => builder => 'debug_';
+has 'debug' => (
+    default => sub {
+        exists($ENV{PERL_PEGEX_DEBUG}) ? $ENV{PERL_PEGEX_DEBUG} :
+        defined($Pegex::Parser::Debug) ? $Pegex::Parser::Debug :
+        0;
+    },
+);
 
-sub debug_ {
-    exists($ENV{PERL_PEGEX_DEBUG}) ? $ENV{PERL_PEGEX_DEBUG} :
-    defined($Pegex::Parser::Debug) ? $Pegex::Parser::Debug :
-    0;
+sub BUILD {
+    my ($self) = @_;
+    my $grammar = $self->grammar;
+    my $receiver = $self->receiver;
+    if ($grammar and not ref $grammar) {
+        $self->grammar($grammar->new);
+    }
+    if ($receiver and not ref $receiver) {
+        $self->receiver($receiver->new);
+    }
 }
 
 sub parse {
-    my $self = shift;
-    $self = $self->new unless ref $self;
+    my ($self, $input, $start_rule) = @_;
 
     die "Usage: " . ref($self) . '->parse($input [, $start_rule]'
-        unless 1 <= @_ and @_ <= 2;
+        unless 2 <= @_ and @_ <= 3;
+
+    $input = Pegex::Input->new(string => $input)->open
+        unless ref $input and UNIVERSAL::isa($input, 'Pegex::Input');
 
-    my $input = (ref $_[0] and UNIVERSAL::isa($_[0], 'Pegex::Input'))
-        ? shift
-        : Pegex::Input->new(shift)->open;
     $self->input($input);
 
     $self->buffer($self->input->read);
 
-    my $grammar = $self->grammar or die "No 'grammar'. Can't parse";
-    if (not ref $grammar) {
-        eval "require $grammar";
-        $self->grammar($grammar->new);
-    }
+    my $grammar = $self->grammar
+        or die "No 'grammar'. Can't parse";
 
-    my $start_rule = shift ||
-        $self->grammar->tree->{'+top'} ||
-        ($self->grammar->tree->{'TOP'} ? 'TOP' : undef)
+    $start_rule ||=
+        $grammar->tree->{'+toprule'} ||
+        ($grammar->tree->{'TOP'} ? 'TOP' : undef)
             or die "No starting rule for Pegex::Parser::parse";
 
-    my $receiver = $self->receiver or die "No 'receiver'. Can't parse";
-    if (not ref $receiver) {
-        eval "require $receiver";
-        $self->receiver($receiver->new);
-    }
+    my $receiver = $self->receiver
+        or die "No 'receiver'. Can't parse";
+
     # Add circular ref and weaken it.
     $self->receiver->parser($self);
     Scalar::Util::weaken($self->receiver->{parser});
@@ -88,8 +108,8 @@ sub parse {
 sub match {
     my ($self, $rule) = @_;
 
-    $self->receiver->initialize($rule)
-        if $self->receiver->can("initialize");
+    $self->receiver->initial($rule)
+        if $self->receiver->can("initial");
 
     my $match = $self->match_next({'.ref' => $rule});
     if (not $match or $self->position < length($self->buffer)) {
@@ -98,8 +118,8 @@ sub match {
     }
     $match = $match->[0];
 
-    $match = $self->receiver->finalize($match, $rule)
-        if $self->receiver->can("finalize");
+    $match = $self->receiver->final($match, $rule)
+        if $self->receiver->can("final");
 
     $match = {$rule => []} unless $match;
 
@@ -133,12 +153,6 @@ sub match_next {
     my ($match, $position, $count, $method) =
         ([], $self->position, 0, "match_$kind");
 
-# XXX Need to rethink this. match_all must be able to complete possible zero
-# width matches at end of stream...
-#     my $return;
-#     while ($position < length($self->{buffer}) and
-#         $return = $self->$method($rule, $next)) {
-
     while (my $return = $self->$method($rule, $next)) {
         $position = $self->position unless $assertion;
         $count++;
@@ -147,11 +161,11 @@ sub match_next {
     }
     if ($max != 1) {
         $match = [$match];
-        $self->position($position);
+        $self->set_position($position);
     }
     my $result = (($count >= $min and (not $max or $count <= $max)) ? 1 : 0)
         ^ ($assertion == -1);
-    $self->position($position)
+    $self->set_position($position)
         if not($result) or $assertion;
 
     $match = [] if $next->{'-skip'};
@@ -169,17 +183,6 @@ sub match_next_with_sep {
     my ($match, $position, $count, $method, $scount, $smin, $smax) =
         ([], $self->position, 0, "match_$kind", 0,
             $self->get_min_max($separator));
-    if ($separator->{'+bok'}) {
-        # TODO refactor with matching code below
-        if (my $return = $self->match_next($separator)) {
-            $position = $self->position;
-            my @return = @$return;
-            if (@return) {
-                @return = @{$return[0]} if $smax != 1;
-                push @$match, @return;
-            }
-        }
-    }
     while (my $return = $self->$method($rule, $next)) {
         $position = $self->position;
         $count++;
@@ -196,23 +199,18 @@ sub match_next_with_sep {
         $match = [$match];
     }
     my $result = (($count >= $min and (not $max or $count <= $max)) ? 1 : 0);
-    $self->revert_back($position)
+    $self->set_position($position)
         if $count == $scount and not $separator->{'+eok'};
 
     $match = [] if $next->{'-skip'};
     return ($result ? $match : 0);
 }
 
-sub revert_back {
-    my ($self, $position) = @_;
-    $self->position($position);
-}
-
 sub match_ref {
     my ($self, $ref, $parent) = @_;
     my $rule = $self->grammar->tree->{$ref};
     $rule ||= $self->can("match_rule_$ref")
-            ? { '.code' => $ref } 
+            ? { '.code' => $ref }
             : die "\n\n*** No grammar support for '$ref'\n\n";
 
     my $trace = (not $rule->{'+asr'} and $self->debug);
@@ -221,40 +219,46 @@ sub match_ref {
     my $match = (ref($rule) eq 'CODE')
         ? $self->$rule()
         : $self->match_next($rule);
-    if (not $match) {
-        $self->trace("not_$ref") if $trace;
-        return 0;
+    if ($match) {
+        $self->trace("got_$ref") if $trace;
+        if (not $rule->{'+asr'} and not $parent->{'-skip'}) {
+            if (my $sub = $self->receiver->can("got_$ref")) {
+                $match = [ $sub->($self->receiver, $match->[0]) ];
+            }
+            elsif (
+                $self->wrap ? not($parent->{'-pass'}) : $parent->{'-wrap'}
+            ) {
+                $match = [ @$match ? { $ref => $match->[0] } : () ];
+            }
+        }
     }
-
-    # Call receiver callbacks
-    $self->trace("got_$ref") if $trace;
-    if (not $rule->{'+asr'} and not $parent->{'-skip'}) {
-        my $callback = "got_$ref";
-        if (my $sub = $self->receiver->can($callback)) {
-            $match = [ $sub->($self->receiver, $match->[0]) ];
-        }
-        elsif ($self->wrap ? not($parent->{'-pass'}) : $parent->{'-wrap'}) {
-            $match = [ @$match ? { $ref => $match->[0] } : () ];
-        }
+    else {
+        $self->trace("not_$ref") if $trace;
+        $match = 0;
     }
 
     return $match;
 }
 
-my $terminater = 0;
+my $xxx_terminator_hack = 0;
+my $xxx_terminator_max = 1000;
+
 sub match_rgx {
     my ($self, $regexp, $parent) = @_;
 
     my $start = pos($self->{buffer}) = $self->position;
+    # XXX A hack for non-terminating grammars. The grammar is bad, but without
+    # this hack, the parser will spin forever. Need to do this right.
     die "Your grammar seems to not terminate at end of stream"
-        if $start >= length $self->{buffer} and $terminater++ > 1000;
+        if $start >= length $self->{buffer}
+            and $xxx_terminator_hack++ > $xxx_terminator_max;
     $self->{buffer} =~ /$regexp/g or return 0;
     my $finish = pos($self->{buffer});
     no strict 'refs';
     my $match = [ map $$_, 1..$#+ ];
     $match = [ $match ] if $#+ > 1;
 
-    $self->position($finish);
+    $self->set_position($finish);
 
     return $match;
 }
@@ -271,7 +275,7 @@ sub match_all {
             $len++;
         }
         else {
-            $self->revert_back($pos);
+            $self->set_position($pos);
             return 0;
         }
     }
@@ -300,9 +304,14 @@ sub match_code {
     return $self->$method();
 }
 
+sub set_position {
+    my ($self, $position) = @_;
+    $self->position($position);
+    $self->farthest($position) if $position > $self->farthest;
+}
+
 sub trace {
-    my $self = shift;
-    my $action = shift;
+    my ($self, $action) = @_;
     my $indent = ($action =~ /^try_/) ? 1 : 0;
     $self->{indent} ||= 0;
     $self->{indent}-- unless $indent;
@@ -311,18 +320,26 @@ sub trace {
     my $snippet = substr($self->buffer, $self->position);
     $snippet = substr($snippet, 0, 30) . "..." if length $snippet > 30;
     $snippet =~ s/\n/\\n/g;
-    print STDERR sprintf("%-30s", $action) . ($indent ? " >$snippet<\n" : "\n");
+    print STDERR sprintf("%-30s", $action) .
+        ($indent ? " >$snippet<\n" : "\n");
 }
 
 sub throw_error {
-    my $self = shift;
-    my $msg = shift;
-    my $line = @{[substr($self->buffer, 0, $self->position) =~ /(\n)/g]} + 1;
-    my $column = $self->position - rindex($self->buffer, "\n", $self->position);
-    my $context = substr($self->buffer, $self->position, 50);
+    my ($self, $msg) = @_;
+    $self->format_error($msg);
+    return 0 unless $self->throw_on_error;
+    require Carp;
+    Carp::croak($self->error);
+}
+
+sub format_error {
+    my ($self, $msg) = @_;
+    my $position = $self->farthest;
+    my $line = @{[substr($self->buffer, 0, $position) =~ /(\n)/g]} + 1;
+    my $column = $position - rindex($self->buffer, "\n", $position);
+    my $context = substr($self->buffer, $position, 50);
     $context =~ s/\n/\\n/g;
-    my $position = $self->position;
-    my $error = <<"...";
+    $self->error(<<"...");
 Error parsing Pegex document:
   msg: $msg
   line: $line
@@ -330,12 +347,7 @@ Error parsing Pegex document:
   context: "$context"
   position: $position
 ...
-    if ($self->throw_on_error) {
-        require Carp;
-        Carp::croak($error);
-    }
-    $@ = $error;
-    return 0;
+    $@ = $self->error;
 }
 
 1;
@@ -343,12 +355,27 @@ Error parsing Pegex document:
 =head1 SYNOPSIS
 
     use Pegex::Parser;
+    use SomeGrammarClass;
+    use SomeReceiverClass;
+
+    my $parser = Pegex::Parser->new(
+        grammar => SomeGrammarClass->new,
+        receiver => SomeReceiverClass->new,
+    );
+
+    my $result = $parser->parse($SomeInputText);
 
 =head1 DESCRIPTION
 
-This is the Pegex module that provides the parsing engine runtime. It has a
-C<parse()> method that applies a grammar to a text that supposedly matches
-that grammar. It also calls the callback methods of its Receiver object.
+Pegex::Parser is the Pegex component that provides the parsing engine runtime.
+It requires a Grammar object and a Receiver object. It's C<parse()> method
+takes an input that is expected to be matched by the grammar, and applies the
+grammar rules to the input. As the grammar is applied the receiver is notified
+of matches. The receiver is free to do whatever it wishes, but often times it
+builds the data into a structure that is commonly known as an AST (Abstract
+Syntax Tree).
 
-Generally this module is not used directly, but is called upon via a
-L<Pegex::Grammar> object.
+When the parse method is complete it returns whatever object the receiver has
+provided as the final result. If the grammar fails to match the input along
+the way, the parse method will throw an error with much information about the
+failure.
